/**
 * @author alteredq / http://alteredqualia.com/
 *
 * Blend two textures
 */

THREE.UnChromakeyShader = {

	uniforms: {

		"tDiffuse1": { type: "t", value: null },
		"tDiffuse2": { type: "t", value: null },
		"mixRatio":  { type: "f", value: 0.5 },
		"opacity":   { type: "f", value: 1.0 },

    tVideo: { type: "t", value: null },
    maxDiff: { type: "f", value: null }

	},

	vertexShader: [

		"varying vec2 vUv;",

		"void main() {",

			"vUv = uv;",
			"gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );",

		"}"

	].join("\n"),

	fragmentShader: [

		"uniform float opacity;",
		"uniform float mixRatio;",

		"uniform sampler2D tDiffuse1;",
		"uniform sampler2D tDiffuse2;",

    "uniform float maxDiff;",
    "uniform sampler2D tVideo;",

		"varying vec2 vUv;",

		"void main() {",

			"vec4 texel1 = texture2D( tDiffuse1, vUv );",
			"vec4 texel2 = texture2D( tDiffuse2, vUv );",
			"vec4 backColor = opacity * mix( texel1, texel2, mixRatio );",
      "vec4 videoColor = texture2D( tVideo, vUv );",
      "float diff = length( backColor.rgb - videoColor.rgb );",
      "if ( diff < maxDiff ) {",
        "discard;",
      "} else {",
        "gl_FragColor = videoColor;",
      "}",

		"}"

	].join("\n")

};
